package com.example.survey.controller;

import com.example.survey.dto.record.*;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.service.RecordService;
import com.example.survey.vo.ResultVO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Pope
 */
@Log4j2
@RestController
@RequestMapping("/record")
public class RecordController {

    @Autowired
    private RecordService recordService;

    @GetMapping("/recordList")
    public ResultVO getRecord(@RequestParam(value = "idNumber", required = false) String idNumber,
                              @RequestParam(value = "state", required = false) String state,
                              @RequestParam(value = "templateName", required = false) String templateName,
                              @RequestParam(value = "templateType", required = false) String templateType,
                              @RequestParam(value = "uuid", required = false) String uuid,
                              @RequestParam(value = "submitTimeGt", required = false) long submitTimeGt,
                              @RequestParam(value = "submitTimeLt", required = false) long submitTimeLt,
                              @RequestParam(value = "projectName", required = false) String projectName,
                              @RequestParam(value = "currentPage", defaultValue = "0") Integer currentPage,
                              @RequestParam(value = "pageSize", defaultValue = "30") Integer pageSize
    ) {
        Map<String, Object> resultMap = new HashMap<>(16, 0.75F);
        resultMap.put("totalCount", recordService.countRecord(idNumber, state, uuid, templateName, templateType,
                                                                submitTimeGt, submitTimeLt, projectName));
        resultMap.put("currentPage", currentPage);
        resultMap.put("pageSize", pageSize);
        resultMap.put("data", recordService.listRecordLimit(idNumber, state, uuid, templateName, templateType,
                            submitTimeGt, submitTimeLt, projectName, currentPage, pageSize));
        ResultVO resultVo = new ResultVO(ResultEnum.SUCCESS);
        resultVo.setData(resultMap);

        return resultVo;
    }

    @GetMapping("/recordValues")
    public ResultVO getRecordValue(@RequestParam("uuid") String uuid
                                  ) {
        ResultVO resultVO = new ResultVO(ResultEnum.SUCCESS);
        resultVO.setData(recordService.getRecordValues(uuid));
        return resultVO;
    }

    @PutMapping("/review")
    public ResultVO reviewRecord(@RequestBody ReviewRecordDTO reviewRecordDTO) {
        recordService.reviewRecord(reviewRecordDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }

    @PutMapping("/record")
    public ResultVO modifyRecord(@RequestBody ModifyRecordDTO modifyRecordDTO) {
        recordService.modifyRecord(modifyRecordDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }

    @PostMapping("/record")
    public ResultVO submitRecord(@RequestBody SubmitRecordDTO submitRecordDTO) {
        recordService.createRecord(submitRecordDTO);

        return new ResultVO(ResultEnum.SUCCESS);
    }

    @DeleteMapping("/record")
    public ResultVO deleteRecord(@RequestBody DeleteRecordDTO deleteRecordDTO){
        recordService.deleteRecord(deleteRecordDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }

    @PostMapping("/file")
    public ResultVO uploadFile(@RequestParam("file") MultipartFile file) {
        ResultVO resultVO = new ResultVO(ResultEnum.SUCCESS);
        resultVO.setData(recordService.uploadFile(file));
        return resultVO;
    }

    @GetMapping("/record2word")
    public void record2word(@RequestParam("uuid")String uuid,
                            HttpServletResponse response){
        recordService.record2word(uuid,response);
    }


    @PutMapping("/metaData")
    public ResultVO modifyMetaData(@RequestBody ModifyMetaDataDTO modifyMetaDataDTO){
        recordService.modifyMetaData(modifyMetaDataDTO);
        return new ResultVO(ResultEnum.SUCCESS);
    }



}
